package de.tudarmstadt.maki.simonstrator.api.component.vehicular.api;

import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkEdge;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkRoute;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.exception.NoAdditionalRouteAvailableException;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.exception.NoExitAvailableException;

/**
 * 
 * @author Tobias Meuser (tobias.meuser@kom.tu-darmstadt.de)
 * @version 1.0 at 06.11.2017
 *
 */
public interface VehicleController {

	/**
	 * This method is used to restrict the shown vehicles to the vehicles in a
	 * certain area.
	 * 
	 * @param pStartX
	 *            the x-start of the observed area
	 * @param pStartY
	 *            the y-start of the observed area
	 * @param pEndX
	 *            the x-end of the observed area
	 * @param pEndY
	 *            the x-end of the observed area
	 */
	void setObservedArea(double pStartX, double pStartY, double pEndX, double pEndY);

	/**
	 * Get the position of a vehicle
	 * 
	 * @param pVehicleID
	 *            the vehicle id of the vehicle (provided by
	 *            {@link #getAllVehicles()})
	 * @return the position of the vehicle
	 */
	Location getVehiclePosition(String pVehicleID);

	/**
	 * Get the position of a vehicle at a certain timestep. The maximum
	 * available timestep can be requested with
	 * {@link #getMaximumAvailablePrediction()}
	 * 
	 * @param pStep
	 *            the timestep at which the position is requested
	 * @param pVehicleID
	 *            the vehicle id of the vehicle (provided by
	 *            {@link #getAllVehicles()})
	 * @return the position of the vehicle
	 */
	Location getVehiclePosition(double pStep, String pVehicleID);

	/**
	 * Perform the next step of the simulator.
	 * 
	 * @return True if successful, false otherwise
	 */
	boolean nextStep();

	/**
	 * Get the ids of all vehicles in the streets.
	 * 
	 * @return a list of all vehicles ids
	 */
	List<String> getAllVehicles();

	/**
	 * Get the ids of all vehicles in the streets at a certain timestep.
	 * 
	 * @param pStep
	 *            the timestep at which the active vehicles are requested
	 * @return a list of all vehicles ids
	 */
	List<String> getAllVehicles(double pStep);

	/**
	 * Returns the current route of the vehicle, only available for
	 * {@link de.tud.kom.p2psim.impl.topology.movement.vehicular.sumo.simulation.controller.traci.TraciSimulationController}
	 * 
	 * @param pVehicleID
	 *            the vehicle id of the vehicle (provided by
	 *            {@link #getAllVehicles()})
	 * @return The current route of the vehicle
	 */
	RoadNetworkRoute getCurrentRoute(String pVehicleID);

	/**
	 * Returns the maximum available prediction
	 * 
	 * @return the maximum available prediction
	 */
	double getMaximumAvailablePrediction();

	/**
	 * Change the route of a vehicle
	 * 
	 * @param pVehicle
	 *            the vehicle id of the vehicle (provided by
	 *            {@link #getAllVehicles()})
	 * @param pRoute
	 *            the new route of the vehicle
	 */
	void rerouteVehicle(String pVehicle, RoadNetworkRoute pRoute);

	/**
	 * Finds a new route of the vehicle towards the given destination
	 * 
	 * @param pVehicle
	 *            the vehicle id of the vehicle (provided by
	 *            {@link #getAllVehicles()})
	 * @return a new route for the vehicle
	 * @throws NoAdditionalRouteAvailableException
	 *             if no additional route is found
	 */
	RoadNetworkRoute findNewRoute(String pVehicle)
			throws NoAdditionalRouteAvailableException;

	/**
	 * Finds a new route of the vehicle towards the given destination
	 * 
	 * @param pVehicle
	 *            the vehicle id of the vehicle (provided by
	 *            {@link #getAllVehicles()})
	 * @param pEdgesToAvoid
	 *            Edges that must not be included in the route
	 * @param pKeepDestination
	 *            should the destination be kept or is route possible
	 * @return a new route for the vehicle
	 * @throws NoAdditionalRouteAvailableException
	 *             if no additional route is found
	 * @throws NoExitAvailableException
	 *             if no exit is found
	 */
	RoadNetworkRoute findNewRoute(String pVehicle,
			List<RoadNetworkEdge> pEdgesToAvoid, boolean pKeepDestination)
					throws NoAdditionalRouteAvailableException,
					NoExitAvailableException;

	/**
	 * get the current step of the simulator
	 * 
	 * @return the step of the simulator
	 */
	double getStep();

	/**
	 * get the start time of the simulator
	 * 
	 * @return the start time of the simulator
	 */
	double getStart();

	/**
	 * Initializes the controller
	 */
	void init();

	/**
	 * get speed of a vehicle speed
	 * 
	 * @param pVehicleID
	 *            the vehicle id of the vehicle (provided by
	 *            {@link #getAllVehicles()})
	 * @return the current speed of the vehicle
	 */
	double getVehicleSpeed(String pVehicleID);

	/**
	 * Stops the vehicle
	 * 
	 * @param pVehicle
	 *            the vehicle id of the vehicle (provided by
	 *            {@link #getAllVehicles()})
	 */
	void stopVehicle(String pVehicle);
}
