/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 *
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.api.EdgeController;

public class RoadNetworkEdge {
	private String _edgeID;
	private double _angle;

	private double _originalMaxSpeed = 0;
	private double _maxSpeed = 0;

	private boolean _isUsableInit = false;
	private boolean _isUsable = false;

	private double _length = Double.NaN;

	private List<RoadNetworkLane> _lanes = new ArrayList<>();

	private List<RoadNetworkEdge> _accessibleEdges = new ArrayList<>();

	private List<RoadNetworkEdge> _incomingEdges = new ArrayList<>();

	private final EdgeController _edgeController;

	private boolean _active = false;

	public RoadNetworkEdge(SerializableRoadNetworkEdge pEdge, EdgeController pEdgeController) {
		_edgeController = pEdgeController;

		_edgeID = pEdge.getEdgeID();
		_angle = pEdge.getAngle();

		for (SerializableRoadNetworkLane lane : pEdge.getLanes()) {
			_lanes.add(new RoadNetworkLane(lane));
		}

		_originalMaxSpeed = pEdge.getMaxSpeed();
		_maxSpeed = _originalMaxSpeed;
	}

	public RoadNetworkEdge(String pEdgeID, double pAngle, EdgeController pEdgeController) {
		_edgeController = pEdgeController;

		_edgeID = pEdgeID;
		_angle = pAngle;
	}

	/**
	 * @param pActive
	 *            the active to set
	 */
	public void setActive(boolean pActive) {
		_active = pActive;
	}

	/**
	 * @return the active
	 */
	public boolean isActive() {
		return _active;
	}

	public List<RoadNetworkLane> getLanes() {
		return _lanes;
	}

	public String getEdgeID() {
		return _edgeID;
	}

	public Location getEdgePosition() {
		return _edgeController.getEdgeMeanPosition(_edgeID);
	}

	public double getLastStepMeanSpeed() {
		return _edgeController.getLastStepMeanSpeed(_edgeID);
	}

	public double getAngle() {
		return _angle;
	}

	public double getOriginalMaxSpeed() {
		return _originalMaxSpeed;
	}

	public int getLaneAmount() {
		return _lanes.size();
	}

	public void addLane(RoadNetworkLane pLane) {
		_lanes.add(pLane);
	}

	public void setOriginalMaxSpeed(double pMaxSpeed) {
		if (pMaxSpeed > _originalMaxSpeed) {
			_originalMaxSpeed = pMaxSpeed;
			_maxSpeed = pMaxSpeed;
		}
	}

	public void setMaxSpeed(double pMaxSpeed) {
		_maxSpeed = pMaxSpeed;
	}

	public double getMaxSpeed() {
		return _maxSpeed;
	}

	public void addConnectedEdge(RoadNetworkEdge pEdge) {
		if (!_accessibleEdges.contains(pEdge)) {
			_accessibleEdges.add(pEdge);
			pEdge.addIncomingEdge(this);
		}
	}

	public void addIncomingEdge(RoadNetworkEdge pEdge) {
		if (!_incomingEdges.contains(pEdge)) {
			_incomingEdges.add(pEdge);
			pEdge.addConnectedEdge(this);
		}
	}

	public List<RoadNetworkEdge> getAccessibleEdges() {
		return Collections.unmodifiableList(_accessibleEdges);
	}

	public List<RoadNetworkEdge> getIncomingEdges() {
		return Collections.unmodifiableList(_incomingEdges);
	}

	public double getLength() {
		if (Double.isNaN(_length)) {
			_length = _edgeController.getEdgeLength(_edgeID);
		}
		return _length;
	}

	public String getDescription() {
		return "id: " + _edgeID + ", lanes: " + _lanes.size() + ", speed limit: " + _originalMaxSpeed;
	}

	@Override
	public String toString() {
		return _edgeID;
	}

	public boolean isInternal() {
		return _edgeID.startsWith(":");
	}

	public void changeEdgeMaxSpeed(double pMaxSpeed) {
		_maxSpeed = pMaxSpeed;
		_edgeController.setEdgeMaxSpeed(getEdgeID(), pMaxSpeed);
	}

	public boolean isUsable() {
		if (!_isUsableInit) {
			_isUsableInit = true;
			_isUsable = _edgeController.isEdgeUsable(getEdgeID());
		}
		return _isUsable;
	}

	public List<List<Location>> getRoadShape() {
		List<List<Location>> laneShapes = new ArrayList<>();

		for (RoadNetworkLane roadNetworkLane : _lanes) {
			List<Location> laneShape = _edgeController.getLaneShape(roadNetworkLane.getLaneID());
			laneShapes.add(laneShape);
		}

		return laneShapes;
	}
}
